@@ -9,7 +9,6 @@ import datetime
 import functools
 import inspect
 import logging
-import uuid
 import warnings
 from collections.abc import (
     AsyncGenerator,
@@ -34,12 +33,14 @@ from typing import (
     TypeVar,
     Union,
     cast,
+    get_type_hints,
     overload,
     runtime_checkable,
 )
 
 from typing_extensions import ParamSpec, TypeGuard, get_args, get_origin
 
+import langsmith._internal._context as _context
 from langsmith import client as ls_client
 from langsmith import run_trees, schemas, utils
 from langsmith._internal import _aiter as aitertools
@@ -52,25 +53,13 @@ if TYPE_CHECKING:
     from langchain_core.runnables import Runnable
 
 LOGGER = logging.getLogger(__name__)
-_PARENT_RUN_TREE = contextvars.ContextVar[Optional[run_trees.RunTree]](
-    "_PARENT_RUN_TREE", default=None
-)
-_PROJECT_NAME = contextvars.ContextVar[Optional[str]]("_PROJECT_NAME", default=None)
-_TAGS = contextvars.ContextVar[Optional[list[str]]]("_TAGS", default=None)
-_METADATA = contextvars.ContextVar[Optional[dict[str, Any]]]("_METADATA", default=None)
-
-
-_TRACING_ENABLED = contextvars.ContextVar[Optional[Union[bool, Literal["local"]]]](
-    "_TRACING_ENABLED", default=None
-)
-_CLIENT = contextvars.ContextVar[Optional[ls_client.Client]]("_CLIENT", default=None)
 _CONTEXT_KEYS: dict[str, contextvars.ContextVar] = {
-    "parent": _PARENT_RUN_TREE,
-    "project_name": _PROJECT_NAME,
-    "tags": _TAGS,
-    "metadata": _METADATA,
-    "enabled": _TRACING_ENABLED,
-    "client": _CLIENT,
+    "parent": _context._PARENT_RUN_TREE,
+    "project_name": _context._PROJECT_NAME,
+    "tags": _context._TAGS,
+    "metadata": _context._METADATA,
+    "enabled": _context._TRACING_ENABLED,
+    "client": _context._CLIENT,
     "replicas": run_trees._REPLICAS,
     "distributed_parent_id": run_trees._DISTRIBUTED_PARENT_ID,
 }
@@ -82,7 +71,19 @@ _OTEL_AVAILABLE: Optional[bool] = None
 
 def get_current_run_tree() -> Optional[run_trees.RunTree]:
     """Get the current run tree."""
-    return _PARENT_RUN_TREE.get()
+    return _context._PARENT_RUN_TREE.get()
+
+
+def set_run_metadata(**metadata: Any) -> None:
+    """Update metadata on the current run tree."""
+    run_tree = get_current_run_tree()
+    if run_tree is None:
+        LOGGER.warning(
+            "No active run tree found. Call `set_run_metadata` inside a traced run."
+        )
+    else:
+        run_tree.metadata.update(metadata)
+    return
 
 
 def get_tracing_context(
@@ -91,12 +92,12 @@ def get_tracing_context(
     """Get the current tracing context."""
     if context is None:
         return {
-            "parent": _PARENT_RUN_TREE.get(),
-            "project_name": _PROJECT_NAME.get(),
-            "tags": _TAGS.get(),
-            "metadata": _METADATA.get(),
-            "enabled": _TRACING_ENABLED.get(),
-            "client": _CLIENT.get(),
+            "parent": _context._PARENT_RUN_TREE.get(),
+            "project_name": _context._PROJECT_NAME.get(),
+            "tags": _context._TAGS.get(),
+            "metadata": _context._METADATA.get(),
+            "enabled": _context._TRACING_ENABLED.get(),
+            "client": _context._CLIENT.get(),
             "replicas": run_trees._REPLICAS.get(),
             "distributed_parent_id": run_trees._DISTRIBUTED_PARENT_ID.get(),
         }
@@ -190,7 +191,7 @@ def ensure_traceable(
     metadata: Optional[Mapping[str, Any]] = None,
     tags: Optional[list[str]] = None,
     client: Optional[ls_client.Client] = None,
-    reduce_fn: Optional[Callable[[Sequence], dict]] = None,
+    reduce_fn: Optional[Callable[[Sequence], Union[dict, str]]] = None,
     project_name: Optional[str] = None,
     process_inputs: Optional[Callable[[dict], dict]] = None,
     process_outputs: Optional[Callable[..., dict]] = None,
@@ -311,7 +312,7 @@ def traceable(
     metadata: Optional[Mapping[str, Any]] = None,
     tags: Optional[list[str]] = None,
     client: Optional[ls_client.Client] = None,
-    reduce_fn: Optional[Callable[[Sequence], dict]] = None,
+    reduce_fn: Optional[Callable[[Sequence], Union[dict, str]]] = None,
     project_name: Optional[str] = None,
     process_inputs: Optional[Callable[[dict], dict]] = None,
     process_outputs: Optional[Callable[..., dict]] = None,
@@ -994,8 +995,8 @@ class trace:
         self.old_ctx = get_tracing_context()
         enabled = utils.tracing_is_enabled(self.old_ctx)
 
-        outer_tags = _TAGS.get()
-        outer_metadata = _METADATA.get()
+        outer_tags = _context._TAGS.get() or _context._GLOBAL_TAGS
+        outer_metadata = _context._METADATA.get() or _context._GLOBAL_METADATA
         client_ = self.client or self.old_ctx.get("client")
         parent_run_ = _get_parent_run(
             {
@@ -1048,11 +1049,11 @@ class trace:
         if enabled is True:
             self.new_run.post()
         if enabled:
-            _TAGS.set(tags_)
-            _METADATA.set(metadata)
-            _PARENT_RUN_TREE.set(self.new_run)
-            _PROJECT_NAME.set(project_name_)
-            _CLIENT.set(client_)
+            _context._TAGS.set(tags_)
+            _context._METADATA.set(metadata)
+            _context._PARENT_RUN_TREE.set(self.new_run)
+            _context._PROJECT_NAME.set(project_name_)
+            _context._CLIENT.set(client_)
 
         return self.new_run
 
@@ -1160,8 +1161,10 @@ def _get_project_name(project_name: Optional[str]) -> Optional[str]:
     prt = _PARENT_RUN_TREE.get()
     return (
         # Maintain tree consistency first
-        _PROJECT_NAME.get()
+        _context._PROJECT_NAME.get()
         or (prt.session_name if prt else None)
+        # Global fallback configured via ls.configure(...)
+        or _context._GLOBAL_PROJECT_NAME
         # fallback to the default for the environment
         or utils.get_tracer_project()
     )
@@ -1393,7 +1396,7 @@ def _get_parent_run(
         return None
     if isinstance(parent, run_trees.RunTree):
         return parent
-    if isinstance(parent, dict):
+    if isinstance(parent, Mapping):
         return run_trees.RunTree.from_headers(
             parent,
             client=langsmith_extra.get("client"),
@@ -1452,14 +1455,14 @@ def _setup_run(
     dangerously_allow_filesystem = container_input.get(
         "dangerously_allow_filesystem", False
     )
-    outer_project = _PROJECT_NAME.get()
+    outer_project = _context._PROJECT_NAME.get()
     langsmith_extra = langsmith_extra or LangSmithExtra()
     name = langsmith_extra.get("name") or container_input.get("name")
-    client_ = langsmith_extra.get("client", client) or _CLIENT.get()
+    client_ = langsmith_extra.get("client", client) or _context._CLIENT.get()
     parent_run_ = _get_parent_run(
         {**langsmith_extra, "client": client_}, kwargs.get("config")
     )
-    project_cv = _PROJECT_NAME.get()
+    project_cv = _context._PROJECT_NAME.get()
     selected_project = (
         project_cv  # From parent trace
         or (
@@ -1467,6 +1470,7 @@ def _setup_run(
         )  # from parent run attempt 2 (not managed by traceable)
         or langsmith_extra.get("project_name")  # at invocation time
         or container_input["project_name"]  # at decorator time
+        or _context._GLOBAL_PROJECT_NAME  # global fallback from ls.configure
         or utils.get_tracer_project()  # default
     )
     reference_example_id = langsmith_extra.get("reference_example_id")
@@ -1487,22 +1491,23 @@ def _setup_run(
             context=copy_context(),
             _token_event_logged=False,
         )
-    id_ = id_ or str(uuid.uuid4())
     signature = inspect.signature(func)
     name_ = name or utils._get_function_name(func)
     extra_inner = _collect_extra(extra_outer, langsmith_extra)
-    outer_metadata = _METADATA.get()
-    outer_tags = _TAGS.get()
+    outer_metadata = _context._METADATA.get() or _context._GLOBAL_METADATA
+    outer_tags = _context._TAGS.get() or _context._GLOBAL_TAGS
     context = copy_context()
     metadata_ = {
         **(langsmith_extra.get("metadata") or {}),
         **(outer_metadata or {}),
     }
-    context.run(_METADATA.set, metadata_)
+    context.run(_context._METADATA.set, metadata_)
     metadata_.update(metadata or {})
     metadata_["ls_method"] = "traceable"
     extra_inner["metadata"] = metadata_
-    inputs, attachments = _get_inputs_and_attachments_safe(signature, *args, **kwargs)
+    inputs, attachments = _get_inputs_and_attachments_safe(
+        signature, *args, func=func, **kwargs
+    )
     invocation_params_fn = container_input.get("invocation_params_fn")
     if invocation_params_fn:
         try:
@@ -1520,7 +1525,7 @@ def _setup_run(
         except BaseException as e:
             LOGGER.error(f"Failed to filter inputs for {name_}: {e}")
     tags_ = (langsmith_extra.get("tags") or []) + (outer_tags or [])
-    context.run(_TAGS.set, tags_)
+    context.run(_context._TAGS.set, tags_)
     tags_ += tags or []
     if parent_run_ is not None:
         new_run = parent_run_.create_child(
@@ -1533,22 +1538,26 @@ def _setup_run(
             attachments=attachments,
         )
     else:
-        new_run = run_trees.RunTree(
-            id=ls_client._ensure_uuid(id_),
-            name=name_,
-            inputs=inputs,
-            run_type=run_type,
-            reference_example_id=ls_client._ensure_uuid(
+        # Create RunTree kwargs conditionally to let RunTree generate id from start_time
+        run_tree_kwargs = {
+            "name": name_,
+            "inputs": inputs,
+            "run_type": run_type,
+            "reference_example_id": ls_client._ensure_uuid(
                 reference_example_id, accept_null=True
             ),
-            project_name=selected_project,  # type: ignore[arg-type]
-            replicas=run_trees._REPLICAS.get(),
-            extra=extra_inner,
-            tags=tags_,
-            client=client_,  # type: ignore
-            attachments=attachments,  # type: ignore
-            dangerously_allow_filesystem=dangerously_allow_filesystem,
-        )
+            "project_name": selected_project,
+            "replicas": run_trees._REPLICAS.get(),
+            "extra": extra_inner,
+            "tags": tags_,
+            "client": client_,
+            "attachments": attachments,
+            "dangerously_allow_filesystem": dangerously_allow_filesystem,
+        }
+        # Only pass id if user explicitly provided one
+        if id_ is not None:
+            run_tree_kwargs["id"] = ls_client._ensure_uuid(id_)
+        new_run = run_trees.RunTree(**cast(Any, run_tree_kwargs))
     if utils.tracing_is_enabled() is True:
         try:
             new_run.post()
@@ -1565,7 +1574,7 @@ def _setup_run(
         context=context,
         _token_event_logged=False,
     )
-    context.run(_PROJECT_NAME.set, response_container["project_name"])
+    context.run(_context._PROJECT_NAME.set, response_container["project_name"])
     context.run(_PARENT_RUN_TREE.set, response_container["new_run"])
     return response_container
 
@@ -1619,38 +1628,69 @@ def _get_inputs_safe(
         return {"args": args, "kwargs": kwargs}
 
 
-def _is_attachment(param: inspect.Parameter) -> bool:
-    return param.annotation == schemas.Attachment or (
+def _is_attachment(param: inspect.Parameter, func: Optional[Callable] = None) -> bool:
+    if param.annotation == schemas.Attachment or (
         get_origin(param.annotation) == Annotated
         and any(arg == schemas.Attachment for arg in get_args(param.annotation))
-    )
+    ):
+        return True
+
+    # try resolving stringified annotations
+    if func is not None and isinstance(param.annotation, str):
+        try:
+            # include_extras=True preserves annotated metadata
+            type_hints = get_type_hints(func, include_extras=True)
+            resolved_annotation = type_hints.get(param.name)
+            if resolved_annotation is not None:
+                return resolved_annotation == schemas.Attachment or (
+                    get_origin(resolved_annotation) == Annotated
+                    and any(
+                        arg == schemas.Attachment
+                        for arg in get_args(resolved_annotation)
+                    )
+                )
+        except (NameError, TypeError, AttributeError):
+            pass
+
+    return False
 
 
-def _attachment_args_helper(signature: inspect.Signature) -> set[str]:
+def _attachment_args_helper(
+    signature: inspect.Signature, func: Optional[Callable] = None
+) -> set[str]:
     return {
-        name for name, param in signature.parameters.items() if _is_attachment(param)
+        name
+        for name, param in signature.parameters.items()
+        if _is_attachment(param, func)
     }
 
 
 @functools.lru_cache(maxsize=1000)
-def _cached_attachment_args(signature: inspect.Signature) -> set[str]:
-    return _attachment_args_helper(signature)
+def _cached_attachment_args(
+    signature: inspect.Signature, func: Optional[Callable] = None
+) -> set[str]:
+    return _attachment_args_helper(signature, func)
 
 
-def _attachment_args(signature: inspect.Signature) -> set[str]:
+def _attachment_args(
+    signature: inspect.Signature, func: Optional[Callable] = None
+) -> set[str]:
     # Caching signatures fails if there's unhashable default values.
     try:
-        return _cached_attachment_args(signature)
+        return _cached_attachment_args(signature, func)
     except TypeError:
-        return _attachment_args_helper(signature)
+        return _attachment_args_helper(signature, func)
 
 
 def _get_inputs_and_attachments_safe(
-    signature: inspect.Signature, *args: Any, **kwargs: Any
+    signature: inspect.Signature,
+    *args: Any,
+    func: Optional[Callable] = None,
+    **kwargs: Any,
 ) -> tuple[dict, schemas.Attachments]:
     try:
         inferred = _get_inputs(signature, *args, **kwargs)
-        attachment_args = _attachment_args(signature)
+        attachment_args = _attachment_args(signature, func)
         if attachment_args:
             inputs, attachments = {}, {}
             for k, v in inferred.items():
@@ -2016,7 +2056,7 @@ def _maybe_create_otel_context(run_tree: Optional[run_trees.RunTree]):
     Returns:
         Context manager for use_span or None if not available.
     """
-    if not run_tree or not utils.is_truish(utils.get_env_var("OTEL_ENABLED")):
+    if not run_tree or not utils.is_env_var_truish("OTEL_ENABLED"):
         return None
     if not _is_otel_available():
         return None
@@ -2047,3 +2087,12 @@ def _maybe_create_otel_context(run_tree: Optional[run_trees.RunTree]):
 
     non_recording_span = NonRecordingSpan(span_context)
     return use_span(non_recording_span)
+
+
+# For backwards compatibility
+_PROJECT_NAME = _context._PROJECT_NAME
+_TAGS = _context._TAGS
+_METADATA = _context._METADATA
+_TRACING_ENABLED = _context._TRACING_ENABLED
+_CLIENT = _context._CLIENT
+_PARENT_RUN_TREE = _context._PARENT_RUN_TREE
