@@ -589,32 +589,6 @@ def test_responses_sync_api():
         == patched_parse.output[0].content[0].parsed
     )
 
-    # Test stream
-    with original_client.responses.stream(
-        input="Say 'foo' then stop.",
-        model="gpt-4o-mini",
-        temperature=0,
-        max_output_tokens=16,
-    ) as original_stream:
-        for _ in original_stream:
-            pass
-        original_full = original_stream.get_final_response()
-    with patched_client.responses.stream(
-        input="Say 'foo' then stop.",
-        model="gpt-4o-mini",
-        temperature=0,
-        max_output_tokens=16,
-    ) as patched_stream:
-        for _ in patched_stream:
-            pass
-        patched_full = patched_stream.get_final_response()
-    original_chunks = list(original_stream)
-    patched_chunks = list(patched_stream)
-    assert len(original_chunks) == len(patched_chunks)
-    for orig, patched in zip(original_chunks, patched_chunks):
-        assert orig.output_text == patched.output_text
-    assert original_full.output_text == patched_full.output_text
-
     time.sleep(0.1)
     for call in mock_session.request.call_args_list:
         assert call[0][0].upper() in ["POST", "GET", "PATCH"]
@@ -687,14 +661,14 @@ async def test_responses_async_api():
         input="Say 'foo' then stop.",
         model="gpt-4o-mini",
         temperature=0,
-        max_output_tokens=32,
+        max_output_tokens=1024,
         text_format=Format,
     )
     patched_parse = await patched_client.responses.parse(
         input="Say 'foo' then stop.",
         model="gpt-4o-mini",
         temperature=0,
-        max_output_tokens=32,
+        max_output_tokens=1024,
         text_format=Format,
     )
 
@@ -705,32 +679,6 @@ async def test_responses_async_api():
         == patched_parse.output[0].content[0].parsed
     )
 
-    # Test stream
-    original_chunks = []
-    patched_chunks = []
-    async with original_client.responses.stream(
-        model="gpt-4o-mini",
-        input=[{"role": "user", "content": "Say 'foo' then stop."}],
-        temperature=0,
-        max_output_tokens=16,
-    ) as original_stream:
-        async for chunk in original_stream:
-            original_chunks.append(chunk)
-        original_full = await original_stream.get_final_response()
-    async with patched_client.responses.stream(
-        model="gpt-4o-mini",
-        input=[{"role": "user", "content": "Say 'foo' then stop."}],
-        temperature=0,
-        max_output_tokens=16,
-    ) as patched_stream:
-        async for chunk in patched_stream:
-            patched_chunks.append(chunk)
-        patched_full = await patched_stream.get_final_response()
-    assert len(original_chunks) == len(patched_chunks)
-    for orig, patched in zip(original_chunks, patched_chunks):
-        assert orig.type == patched.type
-    assert original_full.output_text == patched_full.output_text
-
     time.sleep(0.1)
     for call in mock_session.request.call_args_list:
         assert call[0][0].upper() in ["POST", "GET", "PATCH"]
@@ -738,6 +686,88 @@ async def test_responses_async_api():
     _collect_requests(mock_session, "test_responses_sync_api")
 
 
+def test_responses_stream_sync_api():
+    """Test that the sync responses methods can be traced without errors."""
+    import openai  # noqa
+
+    mock_session = mock.MagicMock()
+    ls_client = langsmith.Client(session=mock_session)
+    original_client = openai.Client()
+    patched_client = wrap_openai(openai.Client(), tracing_extra={"client": ls_client})
+
+    # Test stream
+    with original_client.responses.stream(
+        input="Say 'foo' then stop.",
+        model="gpt-5-nano",
+        max_output_tokens=1024,
+    ) as original_stream:
+        for _ in original_stream:
+            pass
+        original_full = original_stream.get_final_response()
+    with patched_client.responses.stream(
+        input="Say 'foo' then stop.",
+        model="gpt-5-nano",
+        max_output_tokens=1024,
+    ) as patched_stream:
+        for _ in patched_stream:
+            pass
+        patched_full = patched_stream.get_final_response()
+    original_chunks = list(original_stream)
+    patched_chunks = list(patched_stream)
+    assert len(original_chunks) == len(patched_chunks)
+    for orig, patched in zip(original_chunks, patched_chunks):
+        assert orig.output_text == patched.output_text
+    assert original_full.output_text == patched_full.output_text
+
+    time.sleep(0.1)
+    assert mock_session.request.call_count > 1
+    for call in mock_session.request.call_args_list:
+        assert call[0][0].upper() in ["POST", "GET", "PATCH"]
+
+    _collect_requests(mock_session, "test_responses_stream_sync_api")
+
+
+async def test_responses_stream_async_api():
+    """Test that the async responses methods can be traced without errors."""
+    import openai  # noqa
+
+    mock_session = mock.MagicMock()
+    ls_client = langsmith.Client(session=mock_session)
+    original_client = openai.AsyncClient()
+    patched_client = wrap_openai(
+        openai.AsyncClient(), tracing_extra={"client": ls_client}
+    )
+    original_chunks = []
+    patched_chunks = []
+    async with original_client.responses.stream(
+        input="Say 'foo' then stop.",
+        model="gpt-5-nano",
+        max_output_tokens=1024,
+    ) as original_stream:
+        async for chunk in original_stream:
+            original_chunks.append(chunk)
+            pass
+        original_full = await original_stream.get_final_response()
+    async with patched_client.responses.stream(
+        input="Say 'foo' then stop.",
+        model="gpt-5-nano",
+        max_output_tokens=1024,
+    ) as patched_stream:
+        async for chunk in patched_stream:
+            patched_chunks.append(chunk)
+            pass
+        patched_full = await patched_stream.get_final_response()
+    assert len(original_chunks) == len(patched_chunks)
+    assert original_full.output_text == patched_full.output_text
+
+    time.sleep(0.1)
+    assert mock_session.request.call_count > 1
+    for call in mock_session.request.call_args_list:
+        assert call[0][0].upper() in ["POST", "GET", "PATCH"]
+
+    _collect_requests(mock_session, "test_responses_stream_async_api")
+
+
 @pytest.mark.asyncio
 async def test_tool_call_chunking():
     """Test that wrap_openai can reduce tool call chunks when streaming."""
